#ifndef HPSPRIV_H
#define HPSPRIV_H

#include <hpparser.h>
#include <hpstack.h>

// HP data structures
//
// p : prologue
// s : size
// d : data
//
// SAT_DOINT
// pppppsssssd...dg
// data are the digits from lower to upper (e.g. 1966 -> 6691)
// g is sign : - -> 9, + -> 0
//
// SAT_DOREAL
// pppppeeemmmmmmmmmmmmg
// e is exp on 3 digits, signed
// m is mantissa on 12 digits
// g is sign : - -> 9, + -> 0
//
// SAT_DOCSTR
// pppppssssscc...cc
// c is the string, one character using 2 nibbles
//
// SAT_DOCMP
// pppppeeemmmmmmmmmmmmgeeemmmmmmmmmmmmg
// first real (16 nibbles) is real part, second (16 nibbles) is imaginary part
//
// SAT_DOLIST
// pppppo...oddddd
// o is a complete object (with prologue and data)
// d is SAT_SEMI
// Warning : Zint form 1 to 10 are directly encoded with only 5 nibbles containing their address
//           (see ZINT_ below)
//
// SAT_DOMATRIX
// pppppo...oddddd
// o is a complete object (with prologue and data)
// d is SAT_SEMI
// Warning : Zint form 1 to 10 are directly encoded with only 5 nibbles containing their address
//           (see ZINT_ below)
//
// SAT_DOARRY
// pppppssssstttttnnnnnrrrrrccccco...o
// ttttt is the type SAT_ of the elements
// nnnnn is 2 (number of dimensions ?)
// rrrrr is the number of rows
// ccccc is the number of columns
// o is a complete object (with prologue and data)
//
// SAT_DOIDNT
// pppppssc...c
// ss is the length of the symbol name (in characters)
// c...c is the name (2 nibbles per character)
//
// SAT_DOSYMB
// pppppo...oddddd
// o is a complete object (with prologue and data)
// d is SAT_SEMI
//

#ifndef ZINT_m9
#	define ZINT_m9			0x2733F
#endif
#ifndef ZINT_m8
#	define ZINT_m8			0x2734B
#endif
#ifndef ZINT_m7
#	define ZINT_m7			0x27357
#endif
#ifndef ZINT_m6
#	define ZINT_m6			0x27363
#endif
#ifndef ZINT_m5
#	define ZINT_m5			0x2736F
#endif
#ifndef ZINT_m4
#	define ZINT_m4			0x2737B
#endif
#ifndef ZINT_m3
#	define ZINT_m3			0x27387
#endif
#ifndef ZINT_m2
#	define ZINT_m2			0x27393
#endif
#ifndef ZINT_m1
#	define ZINT_m1			0x2739F
#endif
#ifndef ZINT_0
#	define ZINT_0			0x273AB
#endif
#ifndef ZINT_1
#	define ZINT_1			0x273B6
#endif
#ifndef ZINT_2
#	define ZINT_2			0x273C2
#endif
#ifndef ZINT_3
#	define ZINT_3			0x273CE
#endif
#ifndef ZINT_4
#	define ZINT_4			0x273DA
#endif
#ifndef ZINT_5
#	define ZINT_5			0x273E6
#endif
#ifndef ZINT_6
#	define ZINT_6			0x273F2
#endif
#ifndef ZINT_7
#	define ZINT_7			0x273FE
#endif
#ifndef ZINT_8
#	define ZINT_8			0x2740A
#endif
#ifndef ZINT_9
#	define ZINT_9			0x27416
#endif
#ifndef ZINT_10
#	define ZINT_10			0x27422
#endif

#ifndef REAL_m9
#	define REAL_m9			0x2FAB1
#endif
#ifndef REAL_m8
#	define REAL_m8			0x2FA9C
#endif
#ifndef REAL_m7
#	define REAL_m7			0x2FA87
#endif
#ifndef REAL_m6
#	define REAL_m6			0x2FA72
#endif
#ifndef REAL_m5
#	define REAL_m5			0x2FA5D
#endif
#ifndef REAL_m4
#	define REAL_m4			0x2FA48
#endif
#ifndef REAL_m3
#	define REAL_m3			0x2FA33
#endif
#ifndef REAL_m2
#	define REAL_m2			0x2FA1E
#endif
#ifndef REAL_m1
#	define REAL_m1			0x2FA09
#endif
#ifndef REAL_0
#	define REAL_0			0x2F937
#endif
#ifndef REAL_1
#	define REAL_1			0x2F94C
#endif
#ifndef REAL_2
#	define REAL_2			0x2F961
#endif
#ifndef REAL_3
#	define REAL_3			0x2F976
#endif
#ifndef REAL_4
#	define REAL_4			0x2F98B
#endif
#ifndef REAL_5
#	define REAL_5			0x2F9A0
#endif
#ifndef REAL_6
#	define REAL_6			0x2F9B5
#endif
#ifndef REAL_7
#	define REAL_7			0x2F9CA
#endif
#ifndef REAL_8
#	define REAL_8			0x2F9DF
#endif
#ifndef REAL_9
#	define REAL_9			0x2F9F4
#endif
#ifndef MAXREAL
#	define MAXREAL			0x2FAF5
#endif
#ifndef xPI
#	define xPI				0x39AC7
#endif
#ifndef xe
#	define xe				0x39B1E
#endif
#ifndef xINFINITE
#	define xINFINITE		0x08A314
#endif

struct hpobj
{
	int type;
	union
	{
		long long doint;
		unsigned long long dohxs;
		double doreal;
		complex_t docmp;
		char * docstr;
		char *doidnt;
		list_t *dolist;
		array_t *domatrix;
		entry_t *dosymb;
		unsigned dofunc;
	} data;
};

struct list
{
	struct hpobj *obj;
	struct list *prev;
	struct list *next;
};

extern int hps_code (struct hpobj *, unsigned *);
extern int hps_code_int (long long val, unsigned *address);
extern int hps_code_uint (unsigned long long val, unsigned *address);
extern int hps_code_real (double val, unsigned *address);
extern int hps_code_complex (complex_t *val, unsigned *address);
extern int hps_code_str (char *val, unsigned *address);
extern int hps_code_ident (char *val, unsigned *address);
extern int hps_code_list (list_t *val, unsigned *address);
extern int hps_code_array (array_t *val, unsigned *address);
extern int hps_code_symb (entry_t *val, unsigned *address);

extern int hps_decode (unsigned, struct hpobj *, int *);
extern int hps_decode_int (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_uint (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_real (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_complex (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_str (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_ident (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_list (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_array (unsigned address, hpobj_t *obj, int *incr);
extern int hps_decode_symb (unsigned address, hpobj_t *obj, int *incr);

extern int hps_objsize (struct hpobj *obj, int *size);
extern int hps_objsize_type (int, void *data, int *size);

extern int nbdigit (double x, int base);
extern int hps_convert_type (int type);
extern long long BCD2val (unsigned addr, int size, int base);
extern void val2BCD (long long val, unsigned addr, int size, int base);
extern double real2double (unsigned address);
extern void double2real (double val, unsigned address);

extern int hps_get_convert (void);
extern int hps_get_type (int level, int *type);

#endif
