#ifndef HPDIR_H
#define HPDIR_H

 /*!
 * \file hpdir.h
 * Wrappers to new saturn functions that allow to get variables from the RPL directories.
  */
 
//
// INCLUDES
//

#include <saturn.h>			// Needed for #define SAT_ and functions sat_
#include <satdir.h>

//
// CONSTANTS
//

/*!
 * \defgroup hpdretcodes HPDir return codes
 */
 
// @{
 
/*!
 * \brief Return code OK
 *
 */
#define HPD_OK								0
/*!
 * \brief Incorrect parameter passed to the function (e.g. NULL when a pointer is expected)
 *
 */
#define HPD_ERROR_PAR					1
/*!
 * \brief The object in the dir is not of the type asked for
 *
 */
#define HPD_ERROR_TYPE					2
/*!
 * \brief The object was not found
 *
 */
#define HPD_ERROR_NOT_FOUND			3
/*!
 * \brief The type of the RPL object under processing is currently not supported by HPDir
 *
 */
#define HPD_ERROR_NOT_IMPLEMENTED	99
// @}

/*!
 * \defgroup dirrcl Recall objects from directories
 *
 */
//@{

/*!
 * \brief Recalls a DOINT RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches, or if the convert flag is set (see hps_convert) and the type is convertable.
 * The authorized conversions are described below :
 * DOHXS (unsigned long long) -> DOINT (long long)
 * DOREAL (double) -> DOINT (long long)
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_int (char *name, long long *val);
/*!
 * \brief Recalls a DOHXS RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches.
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_uint (char *name, unsigned long long *val);
/*!
 * \brief Recalls a DOREAL RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type match, or if the convert flag is set (see hps_convert) and the type is convertable.
 * The authorized conversions are described below :
 * DOHXS (unsigned long long) -> DOREAL (double)
 * DOINT (long long) -> DOREAL (double)
 * DOCMP (complex_t) -> DOREAL (double) if imaginary part is null
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_real (char *name, double *val);
/*!
 * \brief Recalls a DOCMP RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches.
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_complex (char *name, complex_t *val);
/*!
 * \brief Recalls a DOCSTR RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches.
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_str (char *name, char **val);
/*!
 * \brief Recalls a DOIDENT RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches.
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_ident (char *name, char **val);
/*!
 * \brief Recalls a DOLIST RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches.
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_list (char *name, list_t **val);
/*!
 * \brief Recalls a DOMATRIX or DOARRY RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches.
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_array (char *name, array_t **val);
/*!
 * \brief Recalls a DOSYMB RPL object from the current or an upper directory, and puts its value into the variable passed as argument
 *
 * This function searches for the variable in the current or upper directory.
 * The value is read if the type matches.
 * \param name The name of the object
 * \param val The pointer to the read value
 * \return HPD_ERROR_NOT_FOUND if the variable was not found, HPD_ERROR_TYPE if the type of the object
 * is incompatible, HPD_OK otherwise
 */
extern int hpd_rcl_symb (char *name, entry_t **val);
/*!
 * \brief Returns a string associated with an error code
 *
 * This function returns a string describing the error associated with the error code passed as parameter.
 * \param error The error code
 * \return The string describing the error
 */
extern char *hpd_error (int error);
// @}
#endif
