#ifndef HPSTACK_H
#define HPSTACK_H

#include <stdarg.h>
#include <hpparser.h>

/*!
 * \brief Check if a double is a NaN
 *
 */
#define isnan(x) (((*(unsigned long *)&(x) & 0x7ff00000)== 0x7ff00000) && (((*(unsigned long *)&(x) & 0x000fffff)!= 0) || ((*((unsigned long *)&(x) + 1) & 0xffffffff)!= 0)))

/*!
 * \brief Check if a double is infinite
 *
 */
#define isinf(x) (((*(unsigned long *)&(x) & 0x7ff00000)== 0x7ff00000) && (((*(unsigned long *)&(x) & 0x000fffff)== 0) && ((*((unsigned long *)&(x) + 1) & 0xffffffff)== 0)))

/*!
 * \brief Check if a double is finite
 *
 */
#define isfinite(x) (((*(unsigned long *)&(x) & 0x7ff00000) != 0x7ff00000))

/*!
 * \brief Check if a double is negative
 *
 */
#define isneg (x) (((*(unsigned long *)&(x) & 0x80000000) != 0x80000000))

/*!
 * \mainpage  HPStack
 * Package of access functions to exchange data between the RPL stack and a C application.
 *
 * This set of function allow to pick, pop and push data from/to the RPL stack. It also allows to
 * manipulate the stack (SWAP, DROP, ROT, ROLL, ...).
 * Some C types are implemented as a C counterpart to the RPL types. The RPL types currently
 * addressed are :
 * - DOINT		<-> LONGLONG
 * - DOHXS		<-> ULONGLONG
 * - DOREAL		<-> double
 * - DOCMP		<-> complex_t
 * - DOCSTR		<-> char *
 * - DOIDNT		<-> char *
 * - DOLIST		<-> list_t *
 * - DOARRY		<-> array_t *
 * - DOMATRIX	<-> array_t *
 * - DOSYMB		<-> sumb_t *
 *
 * The elements of a list can be of any of the above types. The elements of an array/matrix can
 * be any authorized types within the list above.
 *
 * The interface provides for high level access functions, which allow to handle easily the most
 * common operations.
 *
 * Some examples :
 *
 * - picking an int from the level 3 of the stack :
 * \code
 *   LONGLONG val;
 *   hps_pick_int (3, &val);
 * \endcode
 *
 * - pushing {1 2.3 "Test"} :
 * \code
 *   list_t *list;
 *   hps_list_create (&list);
 *   hps_list_add_int (list, 1);
 *   hps_list_add_real (list, 2.3);
 *   hps_list_add_str (list, "Test");
 *   hps_push_list (list);
 * \endcode
 *
 * A lower level interface allows to perform more advanced operations.
 *
 * Example :
 * - building a list from the two first levels of the stack :
 * \code
 *   hpobj_t *obj_1;		// Object for storing the 1st data
 *   hpobj_t *obj_2;		// Object for storing the 2d level
 *
 *   // Allocation of the objects
 *   hps_obj_create (&obj_1);
 *   hps_obj_create (&obj_2);
 *
 *   // Picking the objects from the stack
 *   if ((hps_pick (1, obj_1) == HPS_OK) &&
         (hps_pick (2, obj_2) == HPS_OK))
 *   {
 *      list_t *list;	// List to create
 *
 *      // Creation of the list
 *      hps_list_create (&list);
 *      // Adding the objects to the list
 *      hps_list_add (list, obj_2);
 *      hps_list_add (list, obj_1);
 *      // Dropping the objects from the stack
 *      hps_drop ();
 *      hps_drop ();
 *      // Pushing the list onto the stack
 *      hps_push_list (list);
 *   }
 * \endcode
 *
 * Credits :
 *
 * Some major parts of this code are coming or derived from the hplib/saturn files, by Al Borowski, Ingo Blank, Claudio Lapilli.
 * I also found many informations in "Programming in System RPL" by Eduardo Kalinowski, and made a intensive usage of 'nosy' by Jurjen NE Bos to understand the RPL data structures.
 */

 /*!
 * \file hpstack.h
 */

//
// INCLUDES
//

#include <saturn.h>			// Needed for #define SAT_ and functions sat_

//
// CONSTANTS
//

/*!
 * \defgroup hpsretcodes HPStack return codes
 */

// @{

/*!
 * \brief Return code OK
 *
 */
#define HPS_OK								0
/*!
 * \brief Incorrect parameter passed to the function (e.g. NULL when a pointer is expected)
 *
 */
#define HPS_ERROR_PAR					1
/*!
 * \brief Incorrect stack size with regard to the asked action (e.g. empty stack while hps_drop() is called)
 *
 */
#define HPS_ERROR_STACK					2
/*!
 * \brief Overflow data (e.g. DOINT or DOHXS more than 19 digits, so can't be stored within a LONGLONG)
 *
 */
#define HPS_ERROR_OVERFLOW				3
/*!
 * \brief Unable to allocate an object within the Saturn memory
 *
 */
#define HPS_ERROR_SAT_MEMORY			4
/*!
 * \brief Unable to allocate the required data within ARM memory
 *
 */
#define HPS_ERROR_ARM_MEMORY			5
/*!
 * \brief The object in the RPL stack is not of the type asked for (returned by hps_pick_type)
 *
 */
#define HPS_ERROR_TYPE					6
/*!
 * \brief The type of the RPL object under processing is currently not supported by HPStack
 *
 */
#define HPS_ERROR_NOT_IMPLEMENTED	99
// @}

/*!
 * \defgroup sattypes Saturn RPL types
 */

// @{

/*!
 * \brief Saturn data prologue: ZINT (Typical Exact Integer)
 */
#define DOINT 		0x02614
/*!
 * \brief Saturn data prologue: UINT (HEX STRING)
 */
#define DOHXS		0x02A4E
/*!
 * \brief Saturn data prologue: REAL
 */
#define DOREAL		0x02933
/*!
 * \brief Saturn data prologue: COMPLEX
 */
#define DOCMP		0x02977
/*!
 * \brief Saturn data prologue: STRING
 */
#define DOCSTR		0x02A2C
/*!
 * \brief Saturn data prologue: LIST
 */
#define DOLIST		0x02A74
/*!
 * \brief Saturn data prologue: ARRAY
 */
#define DOARRY		0x029E8
/*!
 * \brief Saturn data prologue: MATRIX
 */
#define DOMATRIX		0x02686
/*!
 * \brief Saturn data prologue: ID (variable name)
 */
#define DOIDNT		0x02E48
/*!
 * \brief Saturn data prologue: SYMBOLIC
 */
#define DOSYMB		0x02AB8
/*!
 * \brief Saturn composite object terminator: SEMI
 */
#define SEMI		0x0312B
/*!
 * \brief Generic function identifier for parser (not a Saturn address)
 */
#define DOFUNC		0x01902
/*!
 * \brief Saturn data prologue: XLIB (ROM Pointer)
 */
#define DOROMP		0x02E92

 // @}
/*!
 * \brief The type complex_t is used to store a complex value. Its attributes 're' and 'im' are directly accessible.
 *
 */
typedef struct
{
/*!
 * \brief The real part of the complex number.
 *
 */
	double re;
/*!
 * \brief The imaginary part of the complex number.
 *
 */
	double im;
} complex_t;

/*!
 * \brief The type hpobj_t is used to store any RPL object converted into C structure.
 *
 */
typedef struct hpobj hpobj_t;

/*!
 * \brief The type list_t is used to store a list object converted into C structure.
 *
 */
typedef struct list list_t;

/*!
 * \brief The type array_t is used to store an array/matrix object converted into C structure.
 *
 */
typedef list_t array_t;

/*!
 * \defgroup stackmanag Stack management
 *
 */
//@{
/*!
 * \brief Initializes the access to the RPL stack : NO MORE NEEDED FROM HPGCC 2.0
 *
 * This function MUST be called before using any other hps_ functions. It returns
 * the stack bias that MUST be passed to hps_exit before exiting the main program.
 * \param stack Pointer to the stack bias
 * \return HPS_ERROR_PAR if the parameter is a null pointer, HPS_OK otherwise
 */
extern int hps_init (unsigned *stack);

/*!
 * \brief Finalizes the access to the RPL stack : NO MORE NEEDED FROM HPGCC 2.0
 *
 * This function MUST be called before exiting the main program, with
 * the stack bias returned by hps_init.
 * \param stack The stack bias
 * \return Always HPS_OK
 */
extern int hps_exit (unsigned stack);
//@}

/*!
 * \defgroup stackmanip Stack manipulation
 */
//@{

/*!
 * \brief Gets the depth of the current RPL stack
 *
 * This function gets the depth currently used in the RPL stack.
 * \param depth Pointer to the depth
 * \return HPS_ERROR_PAR if the parameter is a null pointer, HPS_OK otherwise
 */
extern int hps_depth (int *depth);

/*!
 * \brief Gets the available number of levels for the RPL stack
 *
 * This function gets the number of levels that are currently available for the RPL stack.
 * \param avail Pointer to the number of available levels
 * \return HPS_ERROR_PAR if the parameter is a null pointer, HPS_OK otherwise
 */
extern int hps_avail (int *avail);
/*!
 * \brief Drops the top of the current RPL stack
 *
 * This function performs a DROP.
 * \return HPS_ERROR_STACK if the stack is empty, HPS_OK otherwise
 */
extern int hps_drop (void);

/*!
 * \brief Drops the n first objects of the current RPL stack
 *
 * This function performs a DROPN. If there is not enough objects, nothing is dropped.
 * \param levels Number of levels to drop.
 * \return HPS_ERROR_STACK if the stack is too small, HPS_OK otherwise
 */
extern int hps_dropn (int levels);

/*!
 * \brief Swaps the two first levels of the current RPL stack
 *
 * This function performs a SWAP.
 * \return HPS_ERROR_STACK if the stack is too small, HPS_OK otherwise
 */
extern int hps_swap (void);

/*!
 * \brief Rotates the three first levels of the current RPL stack
 *
 * This function performs a ROT.
 * \return HPS_ERROR_STACK if the stack is too small, HPS_OK otherwise
 */
extern int hps_rot (void);

/*!
 * \brief Unrotates the three first levels of the current RPL stack
 *
 * This function performs an UNROT.
 * \return HPS_ERROR_STACK if the stack is too small, HPS_OK otherwise
 */
extern int hps_unrot (void);

/*!
 * \brief Rolls the n first levels of the current RPL stack
 *
 * This function performs a ROLL.
 * \param levels The number of objects to roll.
 * \return HPS_ERROR_STACK if the stack is too small, HPS_OK otherwise
 */
extern int hps_roll (int levels);

/*!
 * \brief Rolls down the n first levels of the current RPL stack
 *
 * This function performs a ROLLD.
 * \param levels The number of objects to roll down.
 * \return HPS_ERROR_STACK if the stack is too small, HPS_OK otherwise
 */
extern int hps_rolld (int levels);
// @}

/*!
 * \defgroup stackaccess Stack Access
 */
//@{

/*!
 * \defgroup highlevelaccess Stack High Level Access
 */
//@{

/*!
 * \brief Authorizes automatic conversion DOHXS -> DOINT -> DOREAL while picking a data from the stack
 *
 * This function authorizes the following conversions when an hps_pick_int or hps_pick_real are performed :
 * - hps_pick_int : DOHXS (ULONGLONG) -> DOINT (LONGLONG)
 * - hps_pick_real : DOHXS (ULONGLONG) -> DOINT (LONGLONG) -> DOREAL (double)
 *
 * \param flag Flag = 1 if conversion are authorized, 0 otherwise
 * \return Always HPS_OK
 */
extern int hps_convert (int flag);

/*!
 * \brief Picks a DOINT RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match, or if the convert flag is set (see hps_convert) and the type is convertable.
 * The authorized conversions are described below :
 * DOHXS (ULONGLONG) -> DOINT (LONGLONG)
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_int (int level, LONGLONG *value);

/*!
 * \brief Picks a DOHXS RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match.
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_uint (int level, ULONGLONG *value);

/*!
 * \brief Picks a DOREAL RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match, or if the convert flag is set (see hps_convert) and the type is convertable.
 * The authorized conversions are described below :
 * DOHXS (ULONGLONG) -> DOINT (LONGLONG) -> DOREAL (double)
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_real (int level, double *value);

/*!
 * \brief Picks a DOCMP RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match.
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_complex (int level, complex_t *value);

/*!
 * \brief Picks a DOCSTR RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match.
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_str (int level, char **value);

/*!
 * \brief Picks a DOLIST RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match.
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_list (int level, list_t **value);

/*!
 * \brief Picks a DOMATRIX or DOARRY RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match.
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_array (int level, array_t **value);

/*!
 * \brief Picks a DOIDNT RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match.
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_ident (int level, char **value);

/*!
 * \brief Picks a DOSYMB RPL object from the stack and puts its value into the variable passed as argument
 *
 * This function checks the type of the RPL object at the asked level against the type passed as argument.
 * The value is read if the type match.
 * \param level The stack level from which pick the object
 * \param value The pointer to the read value
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type of the object
 * at this level does not match the parameter, HPS_OK otherwise
 */
extern int hps_pick_symb (int level, entry_t **value);

/*!
 * \brief Pops an integer from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is an integer, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_int (LONGLONG *value);

/*!
 * \brief Pops an unsigned integer from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is an unsigned integer, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_uint (ULONGLONG *value);

/*!
 * \brief Pops a real from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is a real, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_real (double *value);

/*!
 * \brief Pops a complex from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is a complex, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_complex (complex_t *value);

/*!
 * \brief Pops a string from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is a string, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_str (char **value);

/*!
 * \brief Pops a list from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is a list, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_list (list_t **value);

/*!
 * \brief Pops an array from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is an array, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_array (array_t **value);

/*!
 * \brief Pops an ident from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is an ident, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_ident (char **value);

/*!
 * \brief Pops a symbolic expression from the stack and puts its value into the variable passed as argument
 *
 * This function pops the top of the stack, if it is a symbolic expression, into the variable passed as parameter.
 * \param value Pointer to the variable to fill
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_symb (entry_t **value);

/*!
 * \brief Pushes an integer onto the stack
 *
 * This function pushes a DOINT RPL object, built from the integer value passed as parameter, onto the stack.
 * \param value The integer value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_int (LONGLONG value);

/*!
 * \brief Pushes an unsigned integer onto the stack
 *
 * This function pushes a DOHXS RPL object, built from the unsigned integer value passed as parameter, onto the stack.
 * \param value The unsigned integer value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_uint (ULONGLONG value);

/*!
 * \brief Pushes a real onto the stack
 *
 * This function pushes a DOREAL RPL object, built from the real value passed as parameter, onto the stack.
 * \param value The real value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_real (double value);

/*!
 * \brief Pushes a complex onto the stack
 *
 * This function pushes a DOCMP RPL object, built from the complex value passed as parameter, onto the stack.
 * \param value The pointer to the complex value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_complex (complex_t *value);

/*!
 * \brief Pushes a string onto the stack
 *
 * This function pushes a DOCSTR RPL object, built from the string value passed as parameter, onto the stack.
 * \param value The string value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_str (char *value);

/*!
 * \brief Pushes a list onto the stack
 *
 * This function pushes a DOLIST RPL object, built from the list value passed as parameter, onto the stack.
 * \param value The list value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_list (list_t *value);

/*!
 * \brief Pushes an array onto the stack
 *
 * This function pushes a DOMATRIX RPL object, built from the array value passed as parameter, onto the stack.
 * \param value The array value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_array (array_t *value);

/*!
 * \brief Pushes an ident onto the stack
 *
 * This function pushes a DOIDNT RPL object, built from the string passed as parameter, onto the stack.
 * \param value The ident value to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_ident (char *value);

/*!
 * \brief Pushes a symbolic expression onto the stack
 *
 * This function pushes a DOSYMB RPL object, built from the data passed as parameter, onto the stack.
 * \param value The expression to push
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
 extern int hps_push_symb (entry_t *value);

// @}

/*!
 * \defgroup lowlevelaccess Stack Low Level Access
 */
//@{

/*!
 * \brief Picks a RPL object from the stack and stores it into a C hpobj_t object
 *
 * This function reads a RPL object at the given level of the stack, and stores it into
 * a pre-allocated hpobj_t object.
 * \param level The stack level from which pick the object
 * \param obj Pointer to the target C hpobj_t object
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pick (int level, hpobj_t *obj);

/*!
 * \brief Picks a RPL object of a given type from the stack and stores it into a C hpobj_t object
 *
 * This function reads a RPL object at the given level of the stack, checks its type against the type passed as a parameter,
 * and stores it into a pre-allocated hpobj_t object.
 * \param level The stack level from which pick the object
 * \param type The type used as a filter
 * \param data Pointer to the data of C hpobj_t object
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type ogf the object
 * at this level does not match the parameter, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pick_type (int level, int type, void *data);

/*!
 * \brief Pops a RPL object from the stack and stores it into a C hpobj_t object
 *
 * This function reads the RPL object located at the top of the stack, stores it into
 * a pre-allocated hpobj_t object, and then drops the top of the stack.
 * \param obj Pointer to the target C hpobj_t object
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop (hpobj_t *obj);

/*!
 * \brief Pops a RPL object of a given type from the stack and stores it into a C hpobj_t object
 *
 * This function reads the RPL object located at the top of the stack, checks its type against the type passed as a parameter,
 * stores it into a pre-allocated hpobj_t object, and then drops the top level of the stack.
 * \param type The type used as a filter
 * \param data Pointer to the data of C hpobj_t object
 * \return HPS_ERROR_STACK if the stack is too small, HPS_ERROR_TYPE if the type ogf the object
 * at this level does not match the parameter, HPS_ERROR_NOT_IMPLEMENTED if the object type
 * is not addressed by HPStack, HPS_OK otherwise
 */
extern int hps_pop_type (int type, void *data);

/*!
 * \brief Pushes an object passed as parameter onto the stack
 *
 * This function pushes a RPL object, built from a C hpobj_t object, onto the stack.
 * \param obj The pointer to the object to push onto the stack
 * \return HPS_ERROR_PAR if the pointer parameter is NULL,
 * HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push (hpobj_t *obj);

/*!
 * \brief Pushes an object which type and data area passed as parameter onto the stack
 *
 * This function pushes a RPL object, built from type and data passed as parameter, onto the stack.
 * \param type The type of the object to push onto the stack
 * \param data The data of the object
 * \return HPS_ERROR_SAT_MEMORY if not enough room to allocate the RPL object,
 * HPS_ERROR_NOT_IMPLEMENTED if the object type is not addressed by HPStack,
 * HPS_OK otherwise
 */
extern int hps_push_type (int type, void *data);

// @}
// @}

/*!
 * \brief Returns a string associated with an error code
 *
 * This function returns a string describing the error associated with the error code passed as parameter.
 * \param error The error code
 * \return The string describing the error
 */
extern char *hps_error (int error);

/*!
 * \defgroup cdatamanip C Data Manipulation
 */
//@{

/*!
 * \defgroup listmanip C List Manipulation
 */
//@{

/*!
 * \brief Creates a C list_t object for storing a list
 *
 * This function creates a list that can be later filled unsing hp_list_add... functions.
 * \param list A pointer to the list to create
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_list_create (list_t **list);

/*!
 * \brief Destroys a C list_t object
 *
 * This function destroys recursively a list.
 * \param list A pointer to the list to destroy
 * \return HPS_ERROR_PAR if any parameter error, HPS_OK otherwise
 */
extern int hps_list_destroy (list_t **list);

/*!
 * \brief Adds an element to the end of a list
 *
 * This function adds a C hpobj_t object at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param obj The object to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add (list_t *list, hpobj_t *obj);

/*!
 * \brief Gets the n-th element of a list
 *
 * This function gets a C hpobj_t object that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param obj The object to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_get (list_t *list, int ind, hpobj_t **obj);

/*!
 * \brief Gets the n-th element of a list if an 'int'
 *
 * This function gets an 'int' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'int' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
 */
extern int hps_list_get_int (list_t *list, int ind, LONGLONG *val);

/*!
 * \brief Gets the n-th element of a list if an 'unsigned int'
 *
 * This function gets an 'unsigned int' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'unsigned int' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
 */
extern int hps_list_get_uint (list_t *list, int ind, ULONGLONG *val);

/*!
 * \brief Gets the n-th element of a list if a 'real'
 *
 * This function gets a 'real' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'real' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
 */
extern int hps_list_get_real (list_t *list, int ind, double *val);

/*!
 * \brief Gets the n-th element of a list if a 'string'
 *
 * This function gets a 'string' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'string' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
 */
extern int hps_list_get_str (list_t *list, int ind, char **val);

/*!
 * \brief Gets the n-th element of a list if a 'list'
 *
 * This function gets a 'list' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'list' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
*/
extern int hps_list_get_list (list_t *list, int ind, list_t **val);

/*!
 * \brief Gets the n-th element of a list if a 'array'
 *
 * This function gets a 'array' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'array' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
 */
extern int hps_list_get_array (list_t *list, int ind, array_t **val);

/*!
 * \brief Gets the n-th element of a list if a 'ident'
 *
 * This function gets a 'ident' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'ident' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
 */
extern int hps_list_get_ident (list_t *list, int ind, char **val);

/*!
 * \brief Gets the n-th element of a list if a 'symb'
 *
 * This function gets a 'symb' that is the n-th element of the specified list.
 * \param list The list from which the object has to be got
 * \param ind The index of the object to get
 * \param val The 'symb' to get
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_ERROR_TYPE the n-th element is not an 'int',  HPS_OK otherwise
 */
extern int hps_list_get_symb (list_t *list, int ind, entry_t **val);

/*!
 * \brief Adds an element of the specified type to the end of a list
 *
 * This function adds an element (described by its type and a pointer to its data) at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param type The type of the element to add
 * \param data The pointer to the data of the element to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_type (list_t *list, int type, void *data);

/*!
 * \brief Adds an integer to the end of a list
 *
 * This function adds an integer element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value The integer value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_int (list_t *list, LONGLONG value);

/*!
 * \brief Adds an unsigned integer to the end of a list
 *
 * This function adds an unsigned integer element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value  The unsigned integer value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_uint (list_t *list, ULONGLONG value);

/*!
 * \brief Adds a real to the end of a list
 *
 * This function adds a real element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value The real value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_real (list_t *list, double value);

/*!
 * \brief Adds a complex to the end of a list
 *
 * This function adds a complex element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value The pointer to the complex value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_complex (list_t *list, complex_t *value);

/*!
 * \brief Adds a string to the end of a list
 *
 * This function adds a string element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value The string value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_str (list_t *list, char *value);

/*!
 * \brief Adds a list to the end of a list
 *
 * This function adds a list element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value The list to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_list (list_t *list, list_t *value);

/*!
 * \brief Adds an array to the end of a list
 *
 * This function adds an array element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value The array to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_array (list_t *list, array_t *value);

/*!
 * \brief Adds an ident to the end of a list
 *
 * This function adds an ident element which value is provided at the end of the specified list.
 * \param list The list to which the object has to be added
 * \param value The array to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_list_add_ident (list_t *list, char *value);
// @}

/*!
 * \defgroup arraymanip C Array Manipulation
 */
//@{

/*!
 * \brief Creates a C array_t object for storing an array
 *
 * This function creates an array that can be later filled unsing hp_array_add... functions.
 * \param array A pointer to the array to create
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
 extern int hps_array_create (array_t **array);

/*!
 * \brief Creates a C array_t object from a C array of any dimension
 *
 * This function creates an array and fills it with the content of the C array passed as argument
 * \param array A pointer to the array to create
 * \param data The C array
 * \param type The element type of the C array
 * \param nb The number of dimensions of the C array
 * \param ... The size of each dimension of the C array
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_array_build (array_t **array, void *data, int type, int nb, ...);

/*!
 * \brief Destroys a C array_t object
 *
 * This function destroys recursively an array.
 * \param array A pointer to the array to destroy
 * \return HPS_ERROR_PAR if any parameter error, HPS_OK otherwise
 */
extern int hps_array_destroy (array_t **array);

/*!
 * \brief Gets an element from an array
 *
 * This function gets an element identified by its indices from an array. Th numebr od indices is
 * passd as a parameter, as well as the list of indices.
 * \param array A pointer to the array to destroy
 * \param obj A pointer to the object to get
 * \param nb The number of indices that are passed after
 * \param ... The indices as a list of int
 * \return HPS_ERROR_PAR if any parameter error, HPS_OK otherwise
 */
extern int hps_array_get (array_t *array, hpobj_t **obj, int nb, ...);

/*!
 * \brief Adds an element to the end of a array
 *
 * This function adds a C hpobj_t object at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param obj The object to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add (array_t *array, hpobj_t *obj);

/*!
 * \brief Adds an element of the specified type to the end of an array
 *
 * This function adds an element (described by its type and a pointer to its data) at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param type The type of the element to add
 * \param data The pointer to the data of the element to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the list or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_type (array_t *array, int type, void *data);

/*!
 * \brief Adds an integer to the end of a array
 *
 * This function adds an integer element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value The integer value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_int (array_t *array, LONGLONG value);

/*!
 * \brief Adds an unsigned integer to the end of a array
 *
 * This function adds an unsigned integer element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value  The unsigned integer value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_uint (array_t *array, ULONGLONG value);

/*!
 * \brief Adds a real to the end of a array
 *
 * This function adds a real element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value The real value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_real (array_t *array, double value);

/*!
 * \brief Adds a complex to the end of a array
 *
 * This function adds a complex element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value The pointer to the complex value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_complex (array_t *array, complex_t *value);

/*!
 * \brief Adds a string to the end of a array
 *
 * This function adds a string element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value The string value
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_str (array_t *array, char *value);

/*!
 * \brief Adds a list to the end of a array
 *
 * This function adds a list element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value The list to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_list (array_t *array, list_t *value);

/*!
 * \brief Adds an array to the end of a array
 *
 * This function adds an array element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value The array to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_array (array_t *array, array_t *value);

/*!
 * \brief Adds an ident to the end of a array
 *
 * This function adds an ident element which value is provided at the end of the specified array.
 * \param array The array to which the object has to be added
 * \param value The ident to add
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate the element, HPS_ERROR_PAR if the array or object parameter is incorrect, HPS_OK otherwise
 */
extern int hps_array_add_ident (array_t *array, char *value);
// @}

/*!
 * \defgroup objmanip C Object Manipulation
 */
//@{

/*!
 * \brief Creates a C hpobj_t object that will store a RPL object
 *
 * This function allocates and initializes an hpobj_t object, that will
 * be used for storing an RPL object.
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create (hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object and initializes it with a type and a data
 *
 * This function allocates and initializes an hpobj_t object. The type and data to store
 * are passed as arguments.
 * \param type Type of the RPL object to store
 * \param data Pointer to the value of the object
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_type (int type, void *data, hpobj_t **obj);

/*!
 * \brief Destroys a C hpobj_t object, without destroying the objects pointed by (string, list or array)
 *
 * This function destroys a hpobj_t object. The destruction applies only to the hpobj_t descriptor.
 * If there is any data pointed by (string, list or array), this data is not destroyed.
 * \param obj A pointer to the object (pointer) to destroy
 * \return HPS_ERROR_PAR if any pointer is incorrect, HPS_OK otherwise
 */
extern int hps_obj_destroy (hpobj_t **obj);

/*!
 * \brief Destroys a C hpobj_t object including the objects pointed by (string, list or array)
 *
 * This function destroys a hpobj_t object. The destruction applies to the hpobj_t descriptor and to the
 * data pointed by (string, list or array).
 * \param obj A pointer to the object (pointer) to destroy
 * \return HPS_ERROR_PAR if any pointer is incorrect, HPS_OK otherwise
 */
extern int hps_obj_destroy_all (hpobj_t **obj);

/*!
 * \brief Gets the type and data of a C hpobj_t object
 *
 * This function gets the type and the data from an existing C hpobj_t object.
 * \param obj C hpobj_t object
 * \param type Pointer to the type of the object
 * \param data Generic pointer to the data of the object
 * \return HPS_ERROR_PAR if the object is NULL, HPS_OK otherwise
 */
extern int hps_obj_get (hpobj_t *obj, int * type, void **data);

/*!
 * \brief Creates a C hpobj_t object describing an 'int' data
 *
 * This function allocates an initializes an hpobj_t object containing an 'int' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_int (LONGLONG value, hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object describing an 'unsigned int' data
 *
 * This function allocates an initializes an hpobj_t object containing an 'unsigned int' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_uint (ULONGLONG value, hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object describing a 'real' data
 *
 * This function allocates an initializes an hpobj_t object containing a 'real' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_real (double value, hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object describing a 'complex' data
 *
 * This function allocates an initializes an hpobj_t object containing a 'complex' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_complex (complex_t *value, hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object describing a 'string' data
 *
 * This function allocates an initializes an hpobj_t object containing a 'string' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_str (char *value, hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object describing a 'list' data
 *
 * This function allocates an initializes an hpobj_t object containing a 'list' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_list (list_t *value, hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object describing a 'array' data
 *
 * This function allocates an initializes an hpobj_t object containing a 'array' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_array (array_t *value, hpobj_t **obj);

/*!
 * \brief Creates a C hpobj_t object describing an 'ident' data
 *
 * This function allocates an initializes an hpobj_t object containing an 'ident' data. The value is passed as arguments.
 * \param value Value of the data
 * \param obj A pointer to the object (pointer) to allocate
 * \return HPS_ERROR_ARM_MEMORY if unable to allocate, HPS_OK otherwise
 */
extern int hps_obj_create_ident (char *value, hpobj_t **obj);

/*!
 * \brief Dumps a C hpobj_t object
 *
 * This function is not yet implemented.
 * \param obj The object (pointer) to dump
 * \return A string containing a dump of the object
 */
extern char *hps_dump (hpobj_t *obj);
// @}

// @}
#endif
